package net.knarcraft.bookswithoutborders.utility;

import net.knarcraft.bookswithoutborders.BooksWithoutBorders;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.inventory.meta.BookMeta;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

import static net.knarcraft.bookswithoutborders.utility.InputCleaningHelper.fixName;

/**
 * Helper class for converting books to and from text/yml files
 */
public final class BookToFromTextHelper {

    private BookToFromTextHelper() {
    }

    /**
     * Saves a book's contents to a .yml file
     *
     * @param path         <p>The path of the folder to save to. Must end with a slash</p>
     * @param fileName     <p>The name of the file to load to</p>
     * @param bookMetadata <p>Metadata about the book to save</p>
     * @throws IOException <p>If unable to save the book</p>
     */
    public static void bookToYml(String path, String fileName, BookMeta bookMetadata) throws IOException {
        FileConfiguration bookYml = YamlConfiguration.loadConfiguration(new File(path, "blank"));

        if (bookMetadata.hasTitle()) {
            bookYml.set("Title", bookMetadata.getTitle());
        }
        if (bookMetadata.hasAuthor()) {
            bookYml.set("Author", bookMetadata.getAuthor());
        }
        if (bookMetadata.hasPages()) {
            bookYml.set("Pages", bookMetadata.getPages());
        }
        if (bookMetadata.hasLore()) {
            bookYml.set("Lore", bookMetadata.getLore());
        }

        bookYml.save(path + fileName + ".yml");
    }

    /**
     * Loads a book from a .yml file
     *
     * @param file         <p>The path of the file to load</p>
     * @param bookMetadata <p>Metadata which will be altered with the book's contents</p>
     * @return <p>Metadata for the loaded book</p>
     */
    public static BookMeta bookFromYml(File file, BookMeta bookMetadata) {
        //Handles old style loading
        if (file.getName().substring(file.getName().lastIndexOf(".")).equals(".txt"))
            bookMetadata = bookFromTXT(file.getName(), file, bookMetadata);
        else {
            try {
                FileConfiguration bookYml = YamlConfiguration.loadConfiguration(file);

                bookMetadata.setTitle(bookYml.getString("Title", "Untitled"));
                bookMetadata.setAuthor(bookYml.getString("Author", "Unknown"));
                bookMetadata.setPages(bookYml.getStringList("Pages"));
                bookMetadata.setLore(bookYml.getStringList("Lore"));
            } catch (IllegalArgumentException e) {
                return null;
            }
        }

        return bookMetadata;
    }

    /**
     * Saves a book's contents to a text file
     *
     * @param folderPath   <p>The folder path to save to. Must end with a slash</p>
     * @param fileName     <p>The name of the file to save to</p>
     * @param bookMetadata <p>Metadata about the book to save</p>
     * @throws IOException <p>If unable to save the book</p>
     */
    public static void bookToTXT(String folderPath, String fileName, BookMeta bookMetadata) throws IOException {
        FileWriter fileWriter = new FileWriter(folderPath + fileName + ".txt");
        PrintWriter printWriter = new PrintWriter(fileWriter);
        List<String> pages = bookMetadata.getPages();

        printWriter.println("[Book]");
        for (String page : pages) {
            printWriter.println(page);
        }
        printWriter.close();
    }

    /**
     * Loads a book from a text file
     *
     * @param fileName     <p>The name of the file to load. Used to create author and title</p>
     * @param file         <p>The file to load</p>
     * @param bookMetadata <p>Metadata which will be altered with the book's contents</p>
     * @return <p>Metadata for the loaded book</p>
     */
    private static BookMeta bookFromTXT(String fileName, File file, BookMeta bookMetadata) {
        String author;
        String title;
        if (fileName.contains(BooksWithoutBorders.titleAuthorSeparator)) {
            author = fileName.substring(fileName.indexOf(BooksWithoutBorders.titleAuthorSeparator) + 1, fileName.length() - 4);
            title = fileName.substring(0, fileName.indexOf(BooksWithoutBorders.titleAuthorSeparator));
        } else {
            author = "Unknown";
            title = fileName.substring(0, fileName.length() - 4);
        }

        title = fixName(title, true);

        List<String> rawPages = new ArrayList<>();
        String nullCheck;

        try {
            FileReader fileReader = new FileReader(file);
            BufferedReader bufferedReader = new BufferedReader(fileReader);

            rawPages.add(bufferedReader.readLine());

            if (rawPages.get(0) == null) {
                bufferedReader.close();
                return null;
            }

            //If a book is being loaded its content need not be adjusted
            if (rawPages.get(0).equalsIgnoreCase("[Book]")) {
                rawPages.remove(0);
                nullCheck = bufferedReader.readLine();
                while (nullCheck != null) {
                    rawPages.add(nullCheck);
                    nullCheck = bufferedReader.readLine();
                }
            } else {
                //Adjusts content to page length
                while (rawPages.get(rawPages.size() - 1) != null) {
                    BookFormatter.formatLastPage(rawPages);
                    rawPages.add(bufferedReader.readLine());
                }
            }
            bufferedReader.close();
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }

        List<String> pages = new ArrayList<>(InputCleaningHelper.cleanList(rawPages));

        bookMetadata.setAuthor(author);
        bookMetadata.setTitle(title);
        bookMetadata.setPages(pages);

        return bookMetadata;
    }

}
